# Python script to map selected sites by SecSel 
# using artificially generated data
#
# Usage: python map_results.py result_file_name
#
# *result_file_name is the name of a result file of a single selection trial,
# or a summary file of multiple trials.  The type of the result file is
# infered from the file name.
#
# 2020-08-29  by TAKENAKA, Akio

import re
import sys
import os
import matplotlib.pyplot as plt

# Size of an area.
# To map sites in a area of different size, edit the following.
x_max = 20
y_max = 20


# Result file is to be given as a command line parameter
args = sys.argv

# Show usage if the file name is not given on the command line
if (len(args) < 2):
    print("Usage: python map_results.py  [result_file_name]\n")
    sys.exit()

file_name = args[1]

# Chech if the file exists.
if os.path.exists(file_name) == False:
    print(file_name, " not found.")
    sys.exit()

# Infer the type of the given result file

summary_flag = False # Will be True if the result file is a summary.

# Base name of a summary file ends with "_sum"
m = re.search('_sum\.txt', file_name)

if m:  # Summary file
    summary_flag = True

    # Get colour map
    cmap = 'Reds'
    cm = plt.cm.get_cmap(cmap) 


# Store data to find the maximum selected times when the file is a summary.
site_ids = list()
data = list()


fin = open(file_name, "rt")

fin.readline()  # Skip header line

# Read each line
while True:

    line = fin.readline()  # read each line
    if not line:
        break

    items = re.split('\s+', line[:-1])  # tab separated, but accepts spaces.

    # The first field is a site ID.
    site = items.pop(0)   
    site_ids.append(site)
    
    # If the file is a result of a single run, the second field is the order 
    # the site was selected. Pre-reserved sites are marked by 0.
    # If the file is a summary file, the second field is the times the site is selected.

    val = items.pop(0)  
    data.append(int(val))

fin.close()

# Find maximum times selected in a summary file
n_max = 0

if summary_flag:
    for v in (data):
        if v > n_max:
            n_max = v


# Draw a map

fig = plt.figure()
plt.subplots_adjust(left = 0.1, bottom = 0.06, right = 0.99, top = 0.92)
ax = fig.add_subplot(1, 1, 1)

for site, val in zip(site_ids, data):

    # Will have four tuples, each consisting of x and y coordinates of corners.
    p = list()

    # Get location info from site ID.
    # The first three digits represent the x coordinate
    # and the next three, the y coordinate.
    site_x = int(site[0:3])
    site_y = int(site[3:6])

    p.append((site_x, site_y))
    p.append((site_x + 1, site_y))
    p.append((site_x + 1, site_y + 1))
    p.append((site_x, site_y + 1))

    # For a single run
    if summary_flag == False: 
        # Color of selected sites.
        col = "#8888FF"

        # Color of pre-reserved sites.
        if data == "0":
            col = "#FF8888"

    # For summary file
    else:  

        idx = val / n_max
        if idx > 1:
            idx = 1.0

        col = cm(idx)

    poly = plt.Polygon(p, facecolor = col)
    ax.add_patch(poly)

ax.set_xlim(0, x_max)
ax.set_ylim(0, y_max)

# The title of the map will be the given file name.
plt.title(file_name)

# png file is created.
# Base name of the png file is that of the result file.
base_name = re.sub('\.txt', '', file_name)

img_file = "%s.png" % base_name
plt.savefig(img_file)

# The image is shown on the display
plt.show()
